@@ -1,23 +1,28 @@
 # mypy: ignore-errors
 import os
+import logging
 from collections import defaultdict
 from functools import lru_cache
-from typing import Optional, Union, Callable, List, Dict
+from pathlib import Path
+from typing import Optional, Union, Callable, List, Dict, Set, Any
 
 import numpy as np
 import numpy.typing as npt
 
+logger = logging.getLogger(__name__)
+
 from code_loader.contract.datasetclasses import CustomCallableInterfaceMultiArgs, \
     CustomMultipleReturnCallableInterfaceMultiArgs, ConfusionMatrixCallableInterfaceMultiArgs, CustomCallableInterface, \
     VisualizerCallableInterface, MetadataSectionCallableInterface, PreprocessResponse, SectionCallableInterface, \
     ConfusionMatrixElement, SamplePreprocessResponse, PredictionTypeHandler, InstanceCallableInterface, ElementInstance, \
     InstanceLengthCallableInterface
-from code_loader.contract.enums import MetricDirection, LeapDataType, DatasetMetadataType
+from code_loader.contract.enums import MetricDirection, LeapDataType, DatasetMetadataType, DataStateType
 from code_loader import leap_binder
 from code_loader.contract.mapping import NodeMapping, NodeMappingType, NodeConnection
 from code_loader.contract.visualizer_classes import LeapImage, LeapImageMask, LeapTextMask, LeapText, LeapGraph, \
     LeapHorizontalBar, LeapImageWithBBox, LeapImageWithHeatmap
 from code_loader.inner_leap_binder.leapbinder import mapping_runtime_mode_env_var_mame
+from code_loader.mixpanel_tracker import clear_integration_events, AnalyticsEvent, emit_integration_event_once
 
 import inspect
 import functools
@@ -44,22 +49,36 @@ def _add_mapping_connections(connects_to, arg_names, node_mapping_type, name):
         _add_mapping_connection(user_unique_name, connection_destinations, arg_names, name, node_mapping_type)
 
 
-def integration_test():
+def tensorleap_integration_test():
     def decorating_function(integration_test_function: Callable):
         leap_binder.integration_test_func = integration_test_function
 
         def inner(*args, **kwargs):
             global _called_from_inside_tl_integration_test_decorator
+            # Clear integration test events for new test
+            try:
+                clear_integration_events()
+            except Exception as e:
+                logger.debug(f"Failed to clear integration events: {e}")
             try:
                 _called_from_inside_tl_integration_test_decorator = True
                 ret = integration_test_function(*args, **kwargs)
 
                 try:
                     os.environ[mapping_runtime_mode_env_var_mame] = 'True'
-                    integration_test_function(None, None)
+                    integration_test_function(None, PreprocessResponse(state=DataStateType.training, length=0))
                 except Exception as e:
-                    print(f'Error during integration test: Make sure to disable any non tensorleap decorators '
-                          f'functions before pushing a new TL version')
+                    import traceback
+                    first_tb = traceback.extract_tb(e.__traceback__)[-1]
+                    file_name = Path(first_tb.filename).name
+                    line_number = first_tb.lineno
+                    if isinstance(e, TypeError) and 'is not subscriptable' in str(e):
+                        print(f'Invalid integration code. File {file_name}, line {line_number}: '
+                              f'Please remove this indexing operation usage from the integration test code.')
+                    else:
+                        print(f'Invalid integration code. File {file_name}, line {line_number}: '
+                              f'Integration test is only allowed to call Tensorleap decorators. '
+                              f'Ensure any arithmetics, external library use, Python logic is placed within Tensorleap decoders')
                 finally:
                     if mapping_runtime_mode_env_var_mame in os.environ:
                         del os.environ[mapping_runtime_mode_env_var_mame]
@@ -78,7 +97,8 @@ def _safe_get_item(key):
     except ValueError:
         raise Exception(f'Tensorleap currently supports models with no more then 10 inputs')
 
-def tensorleap_load_model(prediction_types: Optional[List[PredictionTypeHandler]] = None):
+
+def tensorleap_load_model(prediction_types: Optional[List[PredictionTypeHandler]] = []):
     for i, prediction_type in enumerate(prediction_types):
         leap_binder.add_prediction(prediction_type.name, prediction_type.labels, prediction_type.channel_dim, i)
 
@@ -91,10 +111,20 @@ def tensorleap_load_model(prediction_types: Optional[List[PredictionTypeHandler]
             class ModelPlaceholder:
                 def __init__(self):
                     self.model = load_model_func()
+                    # Emit integration test event once per test
+                    try:
+                        emit_integration_event_once(AnalyticsEvent.LOAD_MODEL_INTEGRATION_TEST, {
+                            'prediction_types_count': len(prediction_types)
+                        })
+                    except Exception as e:
+                        logger.debug(f"Failed to emit load_model integration test event: {e}")
 
                 # keras interface
                 def __call__(self, arg):
                     ret = self.model(arg)
+                    if isinstance(ret, list or tuple):
+                        return [r.numpy() for r in ret]
+
                     return ret.numpy()
 
                 def _convert_onnx_inputs_to_correct_type(
@@ -630,6 +660,15 @@ def tensorleap_preprocess():
 
             result = user_function()
             _validate_result(result)
+
+            # Emit integration test event once per test
+            try:
+                emit_integration_event_once(AnalyticsEvent.PREPROCESS_INTEGRATION_TEST, {
+                    'preprocess_responses_count': len(result)
+                })
+            except Exception as e:
+                logger.debug(f"Failed to emit preprocess integration test event: {e}")
+
             return result
 
         return inner
@@ -873,6 +912,15 @@ def tensorleap_input_encoder(name: str, channel_dim=-1, model_input_index=None):
 
             if _called_from_inside_tl_decorator == 0 and _called_from_inside_tl_integration_test_decorator:
                 result = np.expand_dims(result, axis=0)
+                # Emit integration test event once per test
+                try:
+                    emit_integration_event_once(AnalyticsEvent.INPUT_ENCODER_INTEGRATION_TEST, {
+                        'encoder_name': name,
+                        'channel_dim': channel_dim,
+                        'model_input_index': model_input_index
+                    })
+                except Exception as e:
+                    logger.debug(f"Failed to emit input_encoder integration test event: {e}")
 
             return result
 
@@ -958,6 +1006,13 @@ def tensorleap_gt_encoder(name: str):
 
             if _called_from_inside_tl_decorator == 0 and _called_from_inside_tl_integration_test_decorator:
                 result = np.expand_dims(result, axis=0)
+                # Emit integration test event once per test
+                try:
+                    emit_integration_event_once(AnalyticsEvent.GT_ENCODER_INTEGRATION_TEST, {
+                        'encoder_name': name
+                    })
+                except Exception as e:
+                    logger.debug(f"Failed to emit gt_encoder integration test event: {e}")
 
             return result
 
